''Propeller file utilities.spin

' Hardware configuration
'
' Port summary (as of Memorial day 2007)
'-----------------------
'         (40-DIP)     (homemade breadboard usage)
'---------------------------------------------

' these first 7 bits are parallel port outputs
      
' port 0   pin 1     7-segment LED     MOTOR_POWER  = 0 
' port 1   pin 2     7-segment LED     DRIVE_ENABLE = 1
' port 2   pin 3     7-segment LED     HEAD_DIR     = 2
' port 3   pin 4     7-segment LED     HEAD_STEP    = 3 
' port 4   pin 5     7-segment LED     HEAD_SELECT  = 4  
' port 5   pin 6     7-segment LED     WRITE_ENABLE = 5 
' port 6   pin 7     7-segment LED     WRITE_DATA   = 6  
' port 7   pin 8     audio1            LPA bit 8 (no floppy function)
'
' port 8   pin 13     MMC/SD
' port 9   pin 14     MMC/SD
' port 10  pin 15     MMC/SD
' port 11  pin 16     MMC/SD
' port 12  pin 17     middle hct14 pin  6 <- pin  5 <- D pin 15 
' port 13  pin 18     middle hct14 pin 12 <- pin 13 <- D pin 13 floppy READ_DATA
' port 14  pin 19     middle hct14 pin 10 <- pin 11 <- D pin 12 floppy TRACK_ZERO
' port 15  pin 20     middle hct14 pin  8 <- pin  9 <- D pin 10 floppy DISK_INDEX  
'
' port 16  pin 21     LCD 
' port 17  pin 22     LCD
' port 18  pin 23     GPS
' port 19  pin 24     LP-D pin 12 (out to PC)
' port 20  pin 25
' port 21  pin 26     ADC
' port 22  pin 27
' port 23  pin 28     ADC
'
' port 24  pin 33     LP-D pin 13 (out to PC)
' port 25  pin 34     LP-D pin 3 port 1 (in from PC)
' port 26  pin 35     LP-D pin 2 port 0 (in from PC)
' port 27  pin 36     audio2
' port 28  pin 37     I2C EEPROM
' port 29  pin 38     I2C EEPROM
' port 30  pin 39     PGM RS232
' port 31  pin 40     PGM RS232
'
'
'
' External crystal oscillator 5 mHz
'
' On-board seven-segment LED display
'   7 SEGMENT LCD: ports 0-6. 
'   2-line 20-character LCD display

'   Prop port P17 -> pin 22 -> (74HCT14 2 pin 1 -> pin 2)        -> LCD power
'   Prop port P16 -> pin 21 -> (74HCT14 2 pin 9 -> 8 -> ? -> ? ) -> LCD data
'
' GPS 1PPS clock   port  18 = pin23   ' when used, receives 1 PPS square wave from GPS        
'
' MMC/SD card socket
'
' chip select      port 8     ' these three pins: cs, in and clk, are all wired 
' card data input  port 9     ' directly from card to propeller.
' card clock       port 10
' card data output port 11    ' This line has a 10K pullup resistor to +3.3V and a 220 Ohm 
'                             ' resistor in series between it and the propeller pin.
'
' DAC circuit      port 15    ' pin 20
'
' RS232 out to PC  port 30    ' pin 39 usually for Spin programming, but may be used as
'                             ' generic output, as for to send bytes to HyperTerminal
'

CON                                           
                  
   LCD_DATA  = 16 
   LCD_POWER = 17 

   GPS_1PPS  = 18
      
   PC_RS232_OUT = 30
      
   
VAR

byte char

                           
PUB init 

  if (initialized)
    return true
 
  ms_rate := clkfreq/1_000
  us_rate := clkfreq/1_000_000
  
  del := clkfreq >> 4
                                                                         
  dira[LCD_POWER] := 1                           ' This provides power to the LCD.
  dira[LCD_DATA]  := 1                           ' Also set LCD data pin out.
  
  outa[LCD_POWER] := 1                           ' Reset the LCD by turning it off (1 -> 0
  outa[LCD_DATA]  := 0                           ' thru HCT14 inverter). Also set data low.

  delay(800)
    
  outa[LCD_POWER] := 0                           ' Apply power while keeping data low.

  delay(800)
         
  baud := 9600
  bitTime := clkfreq / || baud

  pcBaud := 9600
  pcBitTime := clkfreq / || pcBaud

  clear
                                                                    
  dira[LCD_POWER] := 1     
  dira[LCD_DATA]  := 1

  outa[LCD_POWER] := 0
  outa[LCD_DATA]  := 0

  dira[PC_RS232_OUT] := 1   ' make sure our view of this pin is "output"


'  dira[GPS_1PPS] := 0   
  initialized := true
    
  return true

        
PUB txPC(txByte) | t                             '' transmit a byte to the PC serial port
        
    txByte := ((txByte | $100) << 2)             ' add stop bit, set mode 
    t := cnt                                     ' sync
    repeat 10                                    ' start + eight data bits + stop
      waitcnt(t += pcBitTime)                    ' wait bit time
      outa[PC_RS232_OUT] := ((txByte >>= 1) & 1) ' output bit (true mode)  

    
PUB hexPC(value, digits)                     

  value <<= (8 - digits) << 2
  repeat digits
    txPC(lookupz((value <-= 4) & $F : "0".."9", "A".."F"))


PUB str(strAddr)                               '' transmit z-string at strAddr on LCD  
    repeat strsize(strAddr) - 1                     ' for each character in string
      tx(byte[strAddr++])                        '   write the character
    tx(byte[strAddr]) 


PUB strPC(strAddr)                               '' transmit z-string at strAddr on LCD  
    repeat strsize(strAddr) - 1                     ' for each character in string
      txPC(byte[strAddr++])                        '   write the character
    txPC(byte[strAddr]) 

PUB str1(strAddr)                                 '' transmit z-string at strAddr on LCD
    command(128)
    repeat strsize(strAddr)-1                      ' for each character in string
      tx(byte[strAddr++])                        '   write the character
    tx(byte[strAddr]) 
                      
PUB str2(strAddr)                                 '' transmit z-string at strAddr on LCD
    command(192)
    repeat strsize(strAddr)-1                      ' for each character in string
      tx(byte[strAddr++])                        '   write the character
    tx(byte[strAddr]) 

PUB snsn1(msg1, num1, msg2, num2)                ' 3 ch msg, 4 ch byte, 3 ch msg, 4 ch byte
   
  command(128)
  tx(byte[msg1++])
  tx(byte[msg1++])
  tx(byte[msg1])    
  tx(" ")
  dec(num1,4)
  tx(" ")
  tx(byte[msg2++])
  tx(byte[msg2++])
  tx(byte[msg2])     
  tx(" ")
  dec(num2,4)


PUB sxsx1(msg1, num1, msg2, num2)                ' 3 ch msg, 4 ch byte, 3 ch msg, 4 ch byte
   
  command(128)
  tx(byte[msg1++])
  tx(byte[msg1++])
  tx(byte[msg1])    
  tx(" ")
  hex(num1,4)
  tx(" ")
  tx(byte[msg2++])
  tx(byte[msg2++])
  tx(byte[msg2])     
  tx(" ")
  hex(num2,4)


PUB sxsx2(msg1, num1, msg2, num2)                ' 3 ch msg, 4 ch byte, 3 ch msg, 4 ch byte
   
  command(192)
  tx(byte[msg1++])
  tx(byte[msg1++])
  tx(byte[msg1])    
  tx(" ")
  hex(num1,4)
  tx(" ")
  tx(byte[msg2++])
  tx(byte[msg2++])
  tx(byte[msg2])     
  tx(" ")
  hex(num2,4)


PUB snsn2(msg1, num1, msg2, num2)                ' 4 ch msg, 5 ch byte, 4 ch msg, 4 ch byte
   
  command(192)
  tx(byte[msg1++])
  tx(byte[msg1++])
  tx(byte[msg1])    
  tx(" ")
  dec(num1,4)
  tx(" ")
  tx(byte[msg2++])
  tx(byte[msg2++])
  tx(byte[msg2])     
  tx(" ")
  dec(num2,4)



PUB snsx2(msg1, num1, msg2, num2)                ' 4 ch msg, 5 ch byte, 4 ch msg, 4 ch byte
   
  command(192)
  tx(byte[msg1++])
  tx(byte[msg1++])
  tx(byte[msg1])    
  tx(" ")
  dec(num1,4)
  tx(" ")
  tx(byte[msg2++])
  tx(byte[msg2++])
  tx(byte[msg2])     
  tx(" ")
  hex(num2,8)

  
PUB printAt1(num, start, len)

  command(128 + start)
  hex(num, len)
         


PUB str1n(msg, num, digs) | sz, len           

  sz := strsize(msg)
  len := sz + digs
   
  command(128)
  repeat sz                      
    tx(byte[msg++])                       
  dec(num,digs)

  repeat 20-sz
    tx(" ")
   
  
PUB str1x(msg, num, digs) | sz, len                    

  sz := strsize(msg)
  len := sz + digs
   
  command(128)
  repeat sz                        
    tx(byte[msg++])               
  hex(num,digs)

'  repeat 20-sz
    tx(" ")            


PUB str2n(msg, num, digs) | sz, len                    

  sz := strsize(msg)
  len := sz + digs
   
  command(192)
  repeat sz                       
    tx(byte[msg++])             
  dec(num,digs)

  repeat 20-sz
    tx(" ")
    
'  delay(800)                  


PUB str2x(msg, num, digs) | sz, len                   

  sz := strsize(msg)
  len := sz + digs
   
  command(192)
  repeat sz                     
    tx(byte[msg++])                         
  hex(num,digs)

  repeat 20-sz
    tx(" ")

         
PUB decPC(value, digits) | digit
'
' "dec" works up to about 9 or 10 digits, then breaks down, maybe from math rounding.
' (Propeller greatest 32-bit value: 2147483647)

  if (digits > 9)
    digits := 9

  if (digits < 1)
    digits := 1
    
  if (digits > 8)
    digit := value/100_000_000
    txPC(digit + 48)
    value := value - (100_000_000 * digit)

  if (digits > 7)
    digit := value/10_000_000
    txPC(digit + 48)
    value := value - (10_000_000 * digit)
    
  if (digits > 6)
    digit := value/1_000_000
    txPC(digit + 48)
    value := value - (1_000_000 * digit)

  if (digits > 5)
    digit := value/100_000
    txPC(digit + 48)
    value := value - (100_000 * digit)
    
  if (digits > 4)
    digit := value/10_000
    txPC(digit + 48)
    value := value - (10_000 * digit)
    
  if (digits > 3)
    digit := value/1000
    txPC(digit + 48)
    value := value - (1000 * digit)

  if (digits > 2)
    digit := value/100
    txPC(digit + 48)
    value := value - (100 * digit)
    
  if (digits > 1)
    digit := value/10
    txPC(digit + 48)
    value := value - (10 * digit)
    
  if (digits => 1) 
    txPC(value + 48)
          
PUB dec(value, digits) | digit
'
' "dec" works up to about 9 or 10 digits, then breaks down, maybe from math rounding.
' (Propeller greatest 32-bit value: 2147483647)

  if (digit > 9)
    digit := 9

  if (digit < 1)
    digit := 1
    
  if (digits > 8)
    digit := value/100_000_000
    tx(digit + 48)
    value := value - (100_000_000 * digit)

  if (digits > 7)
    digit := value/10_000_000
    tx(digit + 48)
    value := value - (10_000_000 * digit)
    
  if (digits > 6)
    digit := value/1_000_000
    tx(digit + 48)
    value := value - (1_000_000 * digit)

  if (digits > 5)
    digit := value/100_000
    tx(digit + 48)
    value := value - (100_000 * digit)
    
  if (digits > 4)
    digit := value/10_000
    tx(digit + 48)
    value := value - (10_000 * digit)
    
  if (digits > 3)
    digit := value/1000
    tx(digit + 48)
    value := value - (1000 * digit)

  if (digits > 2)
    digit := value/100
    tx(digit + 48)
    value := value - (100 * digit)
    
  if (digits > 1)
    digit := value/10
    tx(digit + 48)
    value := value - (10 * digit)
    
  if (digits => 1) 
    tx(value + 48)



PUB hex(value, digits)                     

  value <<= (8 - digits) << 2
  repeat digits
    tx(lookupz((value <-= 4) & $F : "0".."9", "A".."F"))

    
PUB clear                                        '' clear LCD screen; pause a second

    command(1)
    delay(500)
        
         
PUB line1                                        '' prepare LCD to display at start of line 1

    command(128)


PUB line2                                        '' prepare LCD to display at start of line 2 

    command(192)

    
PUB line1Ch(pos)                '' position LCD cursor at this position in line 1

  if (pos < 0)
    pos := 0
  if (pos > 20)
    pos := 20
    
  command(128 + pos)


    
PUB line2Ch(ch, pos)                '' position LCD cursor at this position in line 1

  if (pos < 0)
    pos := 0
  if (pos > 20)
    pos := 20
    
  command(192 + pos)
  tx(ch)
          

PUB command(code)                                '' issue Seetron LCD internal commands

  tx(254)
  tx(code)

          
PUB tx(txByte) | t                               '' transmit a byte to the LCD
        
    txByte := ((txByte | $100) << 2)             ' add stop bit, set mode 
    t := cnt                                     ' sync
    repeat 10                                    ' start + eight data bits + stop
      waitcnt(t += bitTime)                      ' wait bit time
      outa[LCD_DATA] := not ((txByte >>= 1) & 1) ' output bit (true mode)  
     

PUB runSlow

  clkset(%0000_0000, 20_000)    
  waitcnt(10000 + cnt)


PUB runFast

  clkset(%0110_0111, 5_000_000)    
  waitcnt(10000 + cnt)



PUB delay(ms) | delt

  delt := (ms * ms_rate) #> 381    
  waitcnt(delt + cnt) 

  
PUB clkCountToMsec(counts)

  Return counts/ms_rate
  
PUB clkCountToUsec(counts)

  Return counts/us_rate
  

DAT

initialized  byte false

del          long $0
ms_rate      long $0
us_rate      long $0  

baud     long $0
bitTime  long $0

pcBaud    long 0
pcBitTime long 0

onems    long $0

              